from collections import defaultdict
from typing import NamedTuple, Iterable

import requests

from wistia.schema import Media, CaptionTrack, Project

from wistia.client import WistiaClient

import logging

log = logging.getLogger(__name__)


class FakeResponse(NamedTuple):
    status_code: int


class DummyWistiaClient(WistiaClient):
    def __init__(self, api_password=""):
        super().__init__(api_password)
        self.session = (
            None
        )  # Make sure we don't hit the API in methods not yet overridden

        self.medias = {}
        self.captions = defaultdict(list)
        self.projects = {}

    def add_dummy_video(self, **kwargs):
        new_media = Media.get_mock_object(overrides=dict(type="Video", **kwargs))
        self.medias[new_media.hashed_id] = new_media

    def list_projects(
        self,
        sort_by=None,  # None:ProjectID,  name, created, or updated
        sort_direction=1,
        page=1,
        per_page=100,
    ) -> Iterable[Project]:
        log.info(
            f"WISTIA API CALL: list_projects("
            f"sort_by={sort_by!r}, sort_direction={sort_direction!r}, "
            f"page={page!r}, per_page={per_page!r})"
        )
        return self.projects.values()

    def list_all_projects(self) -> Iterable[Project]:
        log.info('WISTIA API CALL: list_all_projects()')
        yield from self.projects.values()

    def show_project(self, project_hashed_id: str) -> Project:
        log.info(f"WISTIA API CALL: show_project({project_hashed_id})")
        project = self.projects.get(project_hashed_id, None)
        if not project:
            raise requests.HTTPError(response=FakeResponse(status_code=404))
        return project

    def list_medias(
        self,
        sort_by=None,  # None:ProjectID,  name, created, or updated
        sort_direction=1,
        page=1,
        per_page=100,
        project_id=None,
        name=None,
        media_type=None,
    ) -> Iterable[Media]:
        log.info(
            f"WISTIA API CALL: list_medias("
            f"sort_by={sort_by!r}, sort_direction={sort_direction!r}, "
            f"page={page!r}, per_page={per_page!r}, "
            f"project_id={project_id!r}, name={name!r}, media_type={media_type!r}"
            f")"
        )
        return self.medias.values()

    def show_media(self, wistia_hashed_id: str) -> Media:
        log.info(f"WISTIA API CALL: show_media({wistia_hashed_id})")
        media = self.medias.get(wistia_hashed_id, None)
        if not media:
            raise requests.HTTPError(response=FakeResponse(status_code=404))
        return media

    def show_media_customizations(self, wistia_hashed_id: str) -> dict:
        log.info(f"WISTIA API CALL: show_media_customizations({wistia_hashed_id!r})")
        return {}

    def list_captions(self, wistia_hashed_id: str) -> Iterable[CaptionTrack]:
        log.info(f"WISTIA API CALL: list_captions({wistia_hashed_id!r})")
        media = self.medias.get(wistia_hashed_id, None)
        if not media:
            raise requests.HTTPError(response=FakeResponse(status_code=404))
        return self.captions[wistia_hashed_id]

    def create_captions(
        self,
        wistia_hashed_id: str,
        language_code: str = "eng",
        caption_filename: str = "",
        caption_text: str = "",
    ) -> None:
        log.info(
            f"WISTIA API CALL: create_captions({wistia_hashed_id!r}, {language_code!r}, "
            f"caption_filename={caption_filename!r}, caption_text={caption_text!r})"
        )
        media = self.medias.get(wistia_hashed_id, None)
        if not media:
            raise requests.HTTPError(response=FakeResponse(status_code=404))

        self.captions[wistia_hashed_id].append(
            CaptionTrack(
                {
                    "language": language_code,
                    "text": caption_text,
                    "english_name": "English"
                    if language_code == "eng"
                    else f'"{language_code}"',
                    "native_name": "English"
                    if language_code == "eng"
                    else f'"{language_code}"',
                    "is_draft": False,
                }
            )
        )

    def _captions_for_media_by_language(self, wistia_hashed_id, language_code):
        captions = self.captions[wistia_hashed_id]
        return [track for track in captions if track.language == language_code]

    def show_captions(
        self, wistia_hashed_id, language_code: str = "eng"
    ) -> CaptionTrack:
        log.info(
            f"WISTIA API CALL: show_captions({wistia_hashed_id!r}, "
            f"language_code={language_code!r})"
        )
        matching_captions = self._captions_for_media_by_language(
            wistia_hashed_id, language_code
        )
        if not matching_captions:
            raise requests.HTTPError(response=FakeResponse(status_code=404))
        return matching_captions[0]

    def update_captions(
        self, wistia_hashed_id, language_code, caption_filename="", caption_text=""
    ) -> None:
        log.info(
            f"WISTIA API CALL: update_captions({wistia_hashed_id!r}, {language_code!r}, "
            f"caption_filename={caption_filename!r}, caption_text={caption_text!r})"
        )
        matching_captions = self._captions_for_media_by_language(
            wistia_hashed_id, language_code
        )
        if not matching_captions:
            raise requests.HTTPError(response=FakeResponse(status_code=404))
        matching_captions[0].text = caption_text

    def delete_captions(
        self, wistia_hashed_id: str, language_code: str = "eng"
    ) -> None:
        log.info(f"WISTIA API CALL: delete_captions({wistia_hashed_id!r})")
        matching_captions = self._captions_for_media_by_language(
            wistia_hashed_id, language_code
        )
        if not matching_captions:
            raise requests.HTTPError(response=FakeResponse(status_code=404))
        self.captions[wistia_hashed_id].remove(matching_captions[0])

    def purchase_captions(self, wistia_hashed_id: str) -> None:
        log.info(f"WISTIA API CALL: purchase_captions({wistia_hashed_id!r})")

    def enable_captions_for_media(
        self, wistia_hashed_id: str, enabled: bool = True
    ) -> dict:
        log.info(
            f"WISTIA API CALL: enable_captions_for_media({wistia_hashed_id!r}, enabled={enabled!r})"
        )
        return {}

    def upload_subtitle_file_to_wistia_video(
        self,
        wistia_hashed_id: str,
        subtitle_file_name: str,
        replace=False,
        language_code: str = "eng",
    ) -> None:
        log.info(
            f"WISTIA API CALL: "
            f"upload_subtitle_file_to_wistia_video("
            f"{wistia_hashed_id}, {subtitle_file_name}, replace={replace}, {language_code})"
        )



from os import environ

from wistia.client import WistiaClient
from wistia.dummy import DummyWistiaClient

import logging

log = logging.getLogger(__name__)


def get_wistia_client(password: str = None, client_cls=None) -> WistiaClient:
    # Calling with password='' will get you a DummyWistiaClient
    try:
        # If we're running Django, pull WISTIA_CLIENT_CLASS from settings
        from django.conf import settings

        if client_cls is None:
            client_class_name = getattr(settings, "WISTIA_CLIENT_CLASS", "WistiaClient")
            client_cls = globals()[client_class_name]

        password = password or getattr(settings, "WISTIA_API_PASSWORD", None)
    except ImportError:
        pass

    if password is None:
        password = environ.get("WISTIA_API_PASSWORD")

    if client_cls is None:
        if password is None or password == "":
            log.info("No api password found for wistia client - using dummy client")
            client_cls = DummyWistiaClient
        else:
            client_cls = WistiaClient

    return client_cls(api_password=password)



from schematics import types, models

DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%S%z"


class Asset(models.Model):
    """
    Wrapper for asset objects (embedded in Media objects) for results from
    the Wistia API
    """

    url = types.URLType(
        required=True,
        metadata=dict(description="A direct-access URL to the content of the asset"),
    )
    width = types.IntType(
        required=True,
        metadata=dict(
            description="(optional) The width of this specific asset, if applicable"
        ),
    )
    height = types.IntType(
        required=True,
        metadata=dict(
            description="(optional) The height of this specific asset, if applicable"
        ),
    )
    file_size = types.IntType(
        serialized_name="fileSize",
        required=True,
        metadata=dict(
            description="The size of the asset file that's referenced by url, measured in bytes"
        ),
    )
    content_type = types.StringType(
        serialized_name="contentType",
        required=True,
        metadata=dict(description="The asset's content type"),
    )
    type = types.StringType(
        required=True,
        metadata=dict(
            description="The internal type of the asset, describing how the asset should be used"
        ),
        choices=[
            "OriginalFile",
            "FlashVideoFile",
            "MdFlashVideoFile",
            "HdFlashVideoFile",
            "Mp4VideoFile",
            "MdMp4VideoFile",
            "HdMp4VideoFile",
            "HlsVideoFile",
            "IphoneVideoFile",
            "StoryboardFile",
            "StillImageFile",
            "SwfFile",
            "Mp3AudioFile",
            "LargeImageFile",
        ],
    )


class ProjectReference(models.Model):
    id = types.IntType(required=True, metadata=dict(description=""))
    name = types.StringType(required=True, metadata=dict(description=""))
    hashed_id = types.StringType(required=True, metadata=dict(description=""))


class Thumbnail(models.Model):
    url = types.URLType(required=True, metadata=dict(description=""))
    width = types.IntType(required=True, metadata=dict(description=""))
    height = types.IntType(required=True, metadata=dict(description=""))


class Media(models.Model):
    """ Wrapper for Wistia Media results """

    id = types.IntType(
        required=True,
        metadata=dict(
            description="A unique numeric identifier for the media within the system."
        ),
    )
    name = types.StringType(
        required=True, metadata=dict(description="The display name of the media.")
    )
    hashed_id = types.StringType(
        required=True,
        metadata=dict(description="A unique alphanumeric identifier for this media."),
    )
    description = types.StringType(
        required=True,
        metadata=dict(
            description=(
                "A description for the media which usually appears near the top of the"
                "sidebar on the media’s page"
            )
        ),
    )
    project = types.ModelType(
        ProjectReference,
        required=False,
        metadata=dict(
            description="Information about the project in which the media resides"
        ),
    )
    type = types.StringType(
        required=True,
        metadata=dict(description="A string representing what type of media this is"),
        choices=[
            "Video",
            "Image",
            "Audio",
            "Swf",
            "MicrosoftOfficeDocument",
            "PdfDocument",
            "UnknownType",
        ],
    )
    status = types.StringType(
        required=True,
        default="ready",
        metadata=dict(description="Post upload processing status"),
        choices=["queued", "processing", "ready", "failed"],
    )
    progress = types.FloatType(
        required=False,
        default=1.0,
        metadata=dict(
            description=(
                "(optional) After a file has been uploaded to Wistia, it needs to be"
                "processed before it is available for online viewing. This field is"
                "a floating point value between 0 and 1 that indicates the progress of"
                "that processing."
            )
        ),
    )
    section = types.StringType(
        required=False,
        serialize_when_none=False,
        metadata=dict(
            description=(
                "(optional) The title of the section in which the media appears."
                "This attribute is omitted if the media is not in a section (default)."
            )
        ),
    )
    thumbnail = types.ModelType(
        Thumbnail,
        required=True,
        metadata=dict(
            description="An object representing the thumbnail for this media"
        ),
    )
    duration = types.FloatType(
        required=False,
        metadata=dict(
            description=(
                "(optional) For Audio or Video files, this field specifies the length"
                "(in seconds). For Document files, this field specifies the number of"
                "pages in the document. For other types of media, or if the duration"
                "is unknown, this field is omitted."
            )
        ),
    )
    created = types.DateTimeType(
        required=True,
        serialized_format=DATETIME_FORMAT,
        metadata=dict(description="The date when the media was originally uploaded."),
    )
    updated = types.DateTimeType(
        required=True,
        serialized_format=DATETIME_FORMAT,
        metadata=dict(description="The date when the media was last changed."),
    )
    assets = types.ListType(
        types.ModelType(Asset),
        required=False,
        metadata=dict(description="An array of the assets available for this media"),
    )
    embed_code = types.StringType(
        serialized_name="embedCode",
        required=False,
        metadata=dict(
            description=r"""
            DEPRECATED
            The HTML code that would be used for embedding the media into a web page.
            Please note that in JSON format, all quotes are escaped with a
            backslash (\) character. In XML, angle brackets (< and >) and
            ampersands (&) are converted to their equivalent XML entities
            ("&lt;", "&gt;", and "&amp;" respectively) to prevent XML parser errors.
            """
        ),
    )

    # Undocumented "transcript" field
    # transcript = types.DictType(required=False)


class Project(models.Model):
    """Wrapper for project results."""

    id = types.IntType(
        required=True,
        metadata=dict(
            description="A unique numeric identifier for the project within the system."
        ),
    )
    name = types.StringType(
        required=True, metadata=dict(description="The project's display name.")
    )
    hashed_id = types.StringType(
        required=True,
        metadata=dict(
            description=(
                "A private hashed id, uniquely identifying the project within the"
                "system. Used for playlists and RSS feeds"
            )
        ),
    )
    media_count = types.IntType(
        required=True,
        default=0,
        serialized_name="mediaCount",
        metadata=dict(
            description="The number of different medias that have been uploaded to the project."
        ),
    )
    created = types.DateTimeType(
        required=True,
        serialized_format=DATETIME_FORMAT,
        metadata=dict(description="The date that the project was originally created."),
    )
    updated = types.DateTimeType(
        required=True,
        serialized_format=DATETIME_FORMAT,
        metadata=dict(description="The date that the project was last updated"),
    )
    anonymous_can_upload = types.BooleanType(
        required=True,
        serialized_name="anonymousCanUpload",
        metadata=dict(
            description=(
                "A boolean indicating whether or not anonymous uploads are enabled for the project"
            )
        ),
    )
    anonymous_can_download = types.BooleanType(
        required=True,
        serialized_name="anonymousCanDownload",
        metadata=dict(
            description=(
                "A boolean indicating whether or not anonymous downloads are enabled for this project"
            )
        ),
    )
    public = types.BooleanType(
        required=True,
        metadata=dict(
            description=(
                "A boolean indicating whether the project is available for public (anonymous) viewing"
            )
        ),
    )
    public_id = types.StringType(
        required=False,
        serialized_name="publicId",
        metadata=dict(
            description=(
                "If the project is public, this field contains a string representing the "
                "ID used for referencing the project in public URLs"
            )
        ),
    )
    medias = types.ListType(
        types.ModelType(Media),
        required=False,  # Not present in lists of Projects
        serialize_when_none=False,
        metadata=dict(description="A list of the media associated with a project"),
    )


class CaptionTrack(models.Model):
    language = types.StringType(
        required=True,
        metadata=dict(
            description="A 3 character language code as specified by ISO-639–2"
        ),
    )
    text = types.StringType(
        required=True,
        metadata=dict(
            description="The text of the captions for the specified language in SRT format"
        ),
    )
    english_name = types.StringType(
        required=True, metadata=dict(description="The English name of the language")
    )
    native_name = types.StringType(
        required=True, metadata=dict(description="The native name of the language")
    )
    is_draft = types.BooleanType(
        required=False, metadata=dict(description="Presumably for internal use only")
    )



import logging
from os import path

import pytest
import json

import responses

from wistia import WistiaClient
from wistia.schema import Media

log = logging.getLogger("test_wistia")


EXAMPLE_CAPTIONS = """1
00:00:00,000 --> 00:00:02,000
This is the first caption
"""


@pytest.fixture
def media_list():
    json_string = open(path.join(path.dirname(__file__), "test_media.json")).read()
    return json.loads(json_string)


def test_media_parsing(media_list):
    medias = [Media(media_data) for media_data in media_list]
    assert len(medias) == 2
    assert len(medias[0].assets) == 3


def test_direct_instantiation_of_schema_is_deep(media_list):
    medias = [Media(media_data) for media_data in media_list]
    assert medias[0].thumbnail.url == media_list[0]["thumbnail"]["url"]


@responses.activate
def test_authentication_set_correctly_in_header():
    # Given
    password = "let-me-in"
    client = WistiaClient(api_password=password)
    expected_url = "https://api.wistia.com/v1/medias.json"
    responses.add(responses.GET, url=expected_url, json=[], status=200)

    # When
    client.list_medias()

    # Then
    expected_auth_header = f"Bearer {password}"
    request, response = responses.calls[0]
    observed_auth_header = request.headers["Authorization"]
    assert observed_auth_header == expected_auth_header


@responses.activate
def test_purchase_captions_hits_correct_endpoint():
    media_hashed_id = "12345"
    expected_url = (
        f"https://api.wistia.com/v1/medias/{media_hashed_id}/captions/purchase.json"
    )
    responses.add(responses.POST, url=expected_url, json={}, status=200)

    WistiaClient().purchase_captions(media_hashed_id)
    request, response = responses.calls[0]
    assert request.url == expected_url
    assert request.method == "POST"
    assert request.body is None


@responses.activate
def test_update_captions_hits_correct_endpoint():
    media_hashed_id = "12345"
    expected_url = f"https://api.wistia.com/v1/medias/{media_hashed_id}/captions/eng.json"
    responses.add(responses.PUT, url=expected_url, json={}, status=200)

    WistiaClient().update_captions(media_hashed_id, 'eng', caption_text=EXAMPLE_CAPTIONS)
    request, response = responses.calls[0]
    assert request.url == expected_url
    assert request.method == "PUT"
    assert request.body is not None



"""
Wistia Webhooks

Wistia can send webhooks to your application when certain events happen in your account.

See https://wistia.com/support/developers/webhooks

Headers
=======
The POST request to your webhooks consumer will contain the following headers:

Header	            Description
----------------    ------------------------------------------------------------
Host	            The URL of your consumer, which you provide when configuring a webhook.
User-Agent	        Wistia-Webhooks/{VERSION}. The current version as at 2023-05-10 is 0.0.47.
X-Wistia-Signature	An HMAC hexdigest of the POST body, computed using the secret_key provided
                    when configuring the webhook. You should be using a sha256 HMAC hex digests.
Content-Type	    The output format will always be JSON, so this will always be application/json.
Content-Length	    The byte length of the request body.
"""


import hashlib
import hmac
from datetime import datetime
from typing_extensions import Annotated, Literal
from typing import (
    List,
    Optional,
    Union,
)

import uuid as uuid

# noinspection PyUnresolvedReferences
from pydantic import (
    BaseModel,
    Field,
    ValidationError,
)

media_event_type_names = Literal[
    "media.created",
    "media.processing",
    "media.ready",
    "media.failed",
    "media.updated",
    "media.deleted",
]


viewing_event_type_names = Literal[
    "viewing_session.play",
    "viewing_session.percent_watched",
    "viewing_session.turnstile.converted",
    "viewing_session.call_to_action.converted",
    "viewing_session.annotation.converted",
]


class Thumbnail(BaseModel):
    url: str


class MediaInfo(BaseModel):
    id: str
    name: str
    url: str
    duration: Optional[float]
    thumbnail: Thumbnail


class MediaReference(BaseModel):
    id: str


class MediaPayload(BaseModel):
    media: MediaInfo


class MediaDeletedPayload(MediaPayload):
    media: MediaReference


class MediaUpdatedPayload(BaseModel):
    media: MediaInfo
    previous_attributes: dict = Field(default_factory=dict)


class MediaEvent(BaseModel):
    type: media_event_type_names
    uuid: str
    payload: Union[MediaPayload, MediaUpdatedPayload, MediaDeletedPayload]
    metadata: dict
    generated_at: datetime


class MediaCreatedEvent(MediaEvent):
    type: Literal["media.created"]
    payload: MediaPayload


class MediaProcessingEvent(MediaEvent):
    type: Literal["media.processing"]
    payload: MediaPayload


class MediaReadyEvent(MediaEvent):
    type: Literal["media.ready"]
    payload: MediaPayload


class MediaFailedEvent(MediaEvent):
    type: Literal["media.failed"]
    payload: MediaPayload


class MediaUpdatedEvent(MediaEvent):
    type: Literal["media.updated"]
    payload: MediaUpdatedPayload


class MediaDeletedEvent(MediaEvent):
    type: Literal["media.deleted"]
    payload: MediaDeletedPayload


MediaEvent = Annotated[
    Union[
        MediaCreatedEvent,
        MediaProcessingEvent,
        MediaReadyEvent,
        MediaFailedEvent,
        MediaUpdatedEvent,
        MediaDeletedEvent,
    ],
    Field(discriminator="type"),
]


class HookInfo(BaseModel):
    uuid: uuid.UUID


class EventDelivery(BaseModel):
    hook: HookInfo
    events: List[MediaEvent]


def parse_webhook_event_delivery(event_data: Union[str, bytes, dict]) -> EventDelivery:
    """
    Parse the event data from a Wistia webhook request.
    Raises ValidationError if the event data is invalid.
    :param event_data: Can be a JSON string, a bytes object, or a dict.
    :return: EventDelivery
    """
    if isinstance(event_data, dict):
        return EventDelivery(**event_data)
    else:
        return EventDelivery.parse_raw(event_data)


def compute_signature_hash(request_body: bytes, webhook_secret_key: str) -> str:
    """
    Parse the signature included in the request header and compute the hash of the request body.

    Usage:
    if request.META.get('HTTP_X_WISTIA_SIGNATURE') == compute_signature(
        request_body=request.body,
        secret_key=settings.WISTIA_WEBHOOK_SECRET_KEY,
    ):
        # Authenticated
    """
    return hmac.new(
        key=webhook_secret_key.encode(), msg=request_body, digestmod=hashlib.sha256
    ).hexdigest()


def validate_webhook_signature(request_body: bytes, signature: str, secret_key: str) -> bool:
    """
    Validate the signature included in the request header against the hash of the request body.

    Usage:
    if validate_signature(
        request_body=request.body,
        signature=request.META.get('HTTP_X_WISTIA_SIGNATURE'),
        secret_key=settings.WISTIA_WEBHOOK_SECRET_KEY,
    ):
        # Authenticated
    """
    return hmac.compare_digest(
        signature, compute_signature_hash(request_body, secret_key)
    )



#!/usr/bin/env python
# encoding: utf-8
import logging
import optparse
import sys

# simple CLI for wistia.
import wistia.client

log = logging.getLogger("wistiapy")


def main(args=None):

    DESC = "Wistia python-based command line client."

    if len(sys.argv) < 2:
        sys.argv.append("-h")

    parser = optparse.OptionParser(usage="wistia.py [options]", description=DESC)

    parser.add_option("-c", "--cred", dest="cred", help="your API key", action="store")
    parser.add_option(
        "-p",
        "--projects",
        dest="list_projects",
        help="list all projects",
        action="store_true",
    )
    parser.add_option(
        "-m",
        "--medias",
        dest="list_medias",
        help="list all medias",
        action="store_true",
    )

    (options, args) = parser.parse_args()

    if not options.cred:
        raise Exception("Please supply your credentials with -c KEY")

    # list projects.
    w = wistia.client.WistiaClient("api", options.cred)
    if options.list_projects:
        projects = w.list_projects()
        for p in projects:
            print(f"{p.id}, {p.name}, {p.mediaCount}")

    # list all medias for a project.
    if options.list_medias:
        medias = w.list_medias()
        for m in medias:
            print(f"{m.id}, {m.name}, {m.duration}")

    # create an embed code for media.


if __name__ == "__main__":
    main()



from .__version__ import __version__
from .client import WistiaClient
from .dummy import DummyWistiaClient
from .helpers import get_wistia_client
from .schema import (
    Asset,
    CaptionTrack,
    Media,
    Project,
    ProjectReference,
    Thumbnail,
)
from . import webhooks



import responses
import pytest

from wistia.client import WistiaClient


def generate_http_bearer_auth_string(password):
    return f"Bearer {password}"


@pytest.fixture
def wistia_client():
    password = "letmein"
    return WistiaClient(api_password=password)


@responses.activate
def test_authentication_set_correctly_in_header(wistia_client):
    # Given
    expected_url = 'https://api.wistia.com/v1/medias.json'
    responses.add(responses.GET, url=expected_url, json={}, status=200)

    # When
    wistia_client.list_medias()

    # Then
    expected_auth_header = generate_http_bearer_auth_string("letmein")
    assert expected_auth_header == responses.calls[0].request.headers["Authorization"]


@responses.activate
def test_purchase_captions_hits_correct_endpoint(wistia_client):
    media_hashed_id = "12345"
    expected_url = (
        f"https://api.wistia.com/v1/medias/{media_hashed_id}/captions/purchase.json"
    )
    responses.add(responses.POST, url=expected_url, json={}, status=200)

    wistia_client.purchase_captions(media_hashed_id)
    assert expected_url == responses.calls[0].request.url



import logging
from itertools import count
from typing import Iterable

import requests

from wistia.schema import CaptionTrack, Media, Project

log = logging.getLogger("wistiapy")


class WistiaClient:
    API_BASE_URL = "https://api.wistia.com/v1/"

    def __init__(self, api_password=""):
        # https://wistia.com/support/developers/data-api#authentication
        self.session = requests.Session()
        self.session.headers = {
            "Authorization": f"Bearer {api_password}",
            **self.session.headers
        }

    def request(self, method, rel_path, **kwargs):
        url = f"{self.API_BASE_URL}{rel_path}"
        response = self.session.request(method=method, url=url, **kwargs)
        response.raise_for_status()
        response_data = response.json() if response.text else {}
        return response_data

    def get(self, rel_path: str, params: dict = None):
        return self.request("GET", rel_path, params=params)

    def post(self, rel_path: str, **kwargs):
        return self.request("POST", rel_path, **kwargs)

    def put(self, rel_path: str, **kwargs):
        return self.request("PUT", rel_path, **kwargs)

    def delete(self, rel_path: str):
        return self.request("DELETE", rel_path)

    # Projects

    def list_projects(
        self,
        sort_by=None,  # None:ProjectID,  name, created, or updated
        sort_direction=1,
        page=1,
        per_page=100,
    ) -> Iterable[Project]:
        # https://wistia.com/support/developers/data-api#projects_list
        params = {"page": page, "per_page": per_page}
        if sort_by:
            params["sort_by"] = sort_by
            params["sort_direction"] = sort_direction

        project_list = self.get("projects.json", params=params)
        return [
            Project(project_data, strict=False)
            for project_data in project_list
        ]

    def list_all_projects(self) -> Iterable[Project]:
        log.info("Listing all projects")
        for page in count(start=1):
            next_page_of_projects = self.list_projects(page=page)
            if next_page_of_projects:
                yield from next_page_of_projects
            else:
                return

    def show_project(self, project_hashed_id: str) -> Project:
        # https://wistia.com/support/developers/data-api#projects_show
        rel_path = f"projects/{project_hashed_id}.json"
        project_data = self.get(rel_path)
        return Project(project_data, strict=False)

    # https://wistia.com/support/developers/data-api#projects_create
    # https://wistia.com/support/developers/data-api#projects_update
    # https://wistia.com/support/developers/data-api#projects_delete
    # https://wistia.com/support/developers/data-api#projects_copy

    # Project Sharings
    # https://wistia.com/support/developers/data-api#project_sharings_list
    # https://wistia.com/support/developers/data-api#project_sharings_show
    # https://wistia.com/support/developers/data-api#project_sharings_create
    # https://wistia.com/support/developers/data-api#project_sharings_update
    # https://wistia.com/support/developers/data-api#project_sharings_delete

    # Medias

    def list_medias(
        self,
        sort_by="name",
        sort_direction=1,
        page=1,
        per_page=100,
        project_id=None,
        name=None,
        media_type=None,
    ) -> Iterable[Media]:
        # https://wistia.com/support/developers/data-api#medias_list
        params = {
            "sort_by": sort_by,
            "sort_direction": sort_direction,
            "page": page,
            "per_page": per_page,
        }
        if project_id:
            params["project_id"] = project_id
        if name is not None:
            params["name"] = name
        if media_type is not None:
            params["type"] = media_type

        medias_list = self.get("medias.json", params=params)

        return [
            Media(media_data, strict=False) for media_data in medias_list
        ]

    def show_media(self, wistia_hashed_id: str) -> Media:
        # https://wistia.com/support/developers/data-api#medias_show
        rel_path = f"medias/{wistia_hashed_id}.json"
        media_data = self.get(rel_path)
        return Media(media_data, strict=False)

    # https://wistia.com/support/developers/data-api#medias_update
    # https://wistia.com/support/developers/data-api#medias_delete
    # https://wistia.com/support/developers/data-api#medias_copy
    # https://wistia.com/support/developers/data-api#medias_stats

    # Account
    # https://wistia.com/support/developers/data-api#account

    # Customizations

    def show_media_customizations(self, wistia_hashed_id: str) -> dict:
        # https://wistia.com/support/developers/data-api#customizations_show
        rel_path = f"medias/{wistia_hashed_id}/customizations.json"
        return self.get(rel_path)

    # https://wistia.com/support/developers/data-api#customizations_create
    # https://wistia.com/support/developers/data-api#customizations_update
    # https://wistia.com/support/developers/data-api#customizations_delete

    # Captions

    def list_captions(self, wistia_hashed_id: str) -> Iterable[CaptionTrack]:
        rel_path = f"medias/{wistia_hashed_id}/captions.json"
        caption_list = self.get(rel_path)
        return [
            CaptionTrack(caption_data, strict=False)
            for caption_data in caption_list
        ]

    def create_captions(
        self,
        wistia_hashed_id: str,
        language_code: str = "eng",
        caption_filename: str = "",
        caption_text: str = "",
    ) -> None:
        # https://wistia.com/support/developers/data-api#captions_create
        # Empty 200: OK; 400: already exist; 404: video DNE
        rel_path = f"medias/{wistia_hashed_id}/captions.json"
        if caption_text:
            self.post(
                rel_path, data={"language": language_code, "caption_file": caption_text}
            )
        elif caption_filename:
            with open(caption_filename, "rb") as caption_file:
                self.post(
                    rel_path,
                    data={"language": language_code},
                    files={"caption_file": caption_file},
                )
        else:
            raise ValueError(
                "create_captions requires subtitle_filename or subtitle_text"
            )

    def show_captions(
        self, wistia_hashed_id, language_code: str = "eng"
    ) -> CaptionTrack:
        # https://wistia.com/support/developers/data-api#captions_show
        rel_path = f"medias/{wistia_hashed_id}/captions/{language_code}.json"
        return CaptionTrack(self.get(rel_path))

    def update_captions(
        self, wistia_hashed_id, language_code, caption_filename="", caption_text=""
    ) -> None:
        # https://wistia.com/support/developers/data-api#captions_update
        rel_path = f"medias/{wistia_hashed_id}/captions/{language_code}.json"
        if caption_text:
            self.put(rel_path, data={"caption_file": caption_text})
        elif caption_filename:
            with open(caption_filename, "rb") as caption_file:
                self.put(rel_path, files={"caption_file": caption_file})
        else:
            raise ValueError(
                "update_captions requires subtitle_filename or subtitle_text"
            )

    def delete_captions(
        self, wistia_hashed_id: str, language_code: str = "eng"
    ) -> None:
        # https://wistia.com/support/developers/data-api#captions_delete
        rel_path = f"medias/{wistia_hashed_id}/captions/{language_code}.json"
        self.delete(rel_path)

    def purchase_captions(self, wistia_hashed_id: str) -> None:
        # https://wistia.com/support/developers/data-api#captions_purchase
        rel_path = f"medias/{wistia_hashed_id}/captions/purchase.json"
        self.post(rel_path)

    def enable_captions_for_media(
        self, wistia_hashed_id: str, enabled: bool = True
    ) -> dict:
        # https://wistia.com/support/developers/data-api#customizations_update
        # > If a value is null, then that key will be deleted from the saved customizations.
        # > If it is not null, that value will be set.
        rel_path = f"medias/{wistia_hashed_id}/customizations.json"
        if enabled:
            payload = {"plugin": {"captions-v1": {"onByDefault": False}}}
        else:
            payload = {"plugin": {"captions-v1": None}}

        return self.put(rel_path, json=payload)

    def upload_subtitle_file_to_wistia_video(
        self,
        wistia_hashed_id: str,
        subtitle_file_name: str,
        replace=False,
        language_code: str = "eng",
    ) -> None:
        captions_list = self.list_captions(wistia_hashed_id=wistia_hashed_id)
        replaceable_captions = [
            track for track in captions_list if track.language == language_code
        ]

        if replace and replaceable_captions:
            self.update_captions(
                wistia_hashed_id, language_code, caption_filename=subtitle_file_name
            )
        else:
            self.create_captions(
                wistia_hashed_id, language_code, caption_filename=subtitle_file_name
            )



import uuid
from datetime import datetime, timezone

import pytest

from wistia.webhooks import (
    EventDelivery,
    MediaUpdatedEvent,
    parse_webhook_event_delivery,
)

example_webhook_json_str = """
{
  "hook": {
    "uuid": "a4ab9eb6-ab82-4dae-86f2-29f744f7d031"
  },
  "events": [
    {
      "uuid": "fc53f8f78b67d04d455029813f8ec1ef",
      "type": "media.failed",
      "payload": {
        "media": {
          "duration": 40.207,
          "id": "vpe2p82q64",
          "name": "Lenny Delivers Video!",
          "thumbnail": {
            "url": "http://embed.wistia.com/deliveries/e96ea382701bc172657d90bf269211beddbf6f7f.jpg?image_crop_resized=200x120"
          },
          "url": "https://harper.wistia.com/medias/vpe2p82q64"
        }
      },
      "metadata": {
        "account_id": "0sxav1wj8o"
      },
      "generated_at": "2020-03-31T21:56:45Z"
    }
  ]
}"""


def test_webhook_models_parse_event_data_from_json_string():
    parsed_event_delivery = parse_webhook_event_delivery(example_webhook_json_str)
    assert parsed_event_delivery.hook.uuid == uuid.UUID(
        "a4ab9eb6-ab82-4dae-86f2-29f744f7d031"
    )
    assert len(parsed_event_delivery.events) == 1
    event = parsed_event_delivery.events[0]
    assert event.uuid == "fc53f8f78b67d04d455029813f8ec1ef"
    assert event.type == "media.failed"
    assert event.payload.media.id == "vpe2p82q64"
    assert event.payload.media.name == "Lenny Delivers Video!"
    assert event.payload.media.duration == 40.207
    assert (
        event.payload.media.thumbnail.url
        == "http://embed.wistia.com/deliveries/e96ea382701bc172657d90bf269211beddbf6f7f.jpg?image_crop_resized=200x120"
    )
    assert event.payload.media.url == "https://harper.wistia.com/medias/vpe2p82q64"
    assert event.metadata["account_id"] == "0sxav1wj8o"
    assert event.generated_at == datetime(2020, 3, 31, 21, 56, 45, tzinfo=timezone.utc)


delivery_template = {
    "hook": {
        "uuid": "a4ab9eb6-ab82-4dae-86f2-29f744f7d031"
    },
    "events": [],
}

media_event_data_template = {
    "uuid": "fc53f8f78b67d04d455029813f8ec1ef",
    "payload": {
        "media": {
            "id": "vpe2p82q64",
            "name": "Lenny Delivers Video!",
            "url": "https://harper.wistia.com/medias/vpe2p82q64",
            "duration": 40.207,
            "thumbnail": {
                "url": "http://embed.wistia.com/deliveries/e96ea382701bc172657d90bf269211beddbf6f7f.jpg?image_crop_resized=200x120"
            },
        }
    },
    "metadata": {"account_id": "0sxav1wj8o"},
    "generated_at": "2020-03-31T21:56:45Z",
}


media_created_event_data = {
    **media_event_data_template,
    "type": "media.created",
}

media_processing_event_data = {
    **media_event_data_template,
    "type": "media.processing",
}

media_ready_event_data = {
    **media_event_data_template,
    "type": "media.ready",
}

media_failed_event_data = {
    **media_event_data_template,
    "type": "media.failed",
}


media_updated_event_data = {
    **media_event_data_template,
    "type": "media.updated",
    "payload": {
        **media_event_data_template['payload'],
        "previous_attributes": {
            "thumbnail": {
                "url": "http://embed.wistia.com/deliveries/e04d7729a8b83f4fdd52e107b980f5594359bb45.jpg?image_crop_resized=200x120"
            }
        },
    },
}


media_deleted_event_data = {
    **media_event_data_template,
    "type": "media.deleted",
    "payload": {"media": {"id": "f5diqltruh"}},
}


def test_media_created_event():
    media_created_event_delivery_data = {**delivery_template, "events": [media_created_event_data]}
    # Duration tends to be `null` for creation events, sometimes also updated events
    media_created_event_delivery_data['events'][0]['payload']['media']['duration'] = None
    delivery = EventDelivery(**media_created_event_delivery_data)
    assert delivery.hook.uuid == uuid.UUID(delivery_template['hook']['uuid'])
    event = delivery.events[0]
    assert event.type == "media.created"
    assert event.uuid == media_created_event_data['uuid']
    assert event.payload.media.id == media_created_event_data['payload']['media']['id']
    assert event.payload.media.duration is None
    assert event.metadata['account_id'] == media_created_event_data['metadata']['account_id']
    assert event.generated_at == datetime.strptime(
        media_created_event_data['generated_at'], "%Y-%m-%dT%H:%M:%SZ"
    ).replace(tzinfo=timezone.utc)


def test_media_updated_event_does_not_require_previous_attributes():
    updated_event_with_previous_attributes = MediaUpdatedEvent.parse_obj(media_updated_event_data)
    assert 'thumbnail' in updated_event_with_previous_attributes.payload.previous_attributes
    updated_event_without_previous_attributes = MediaUpdatedEvent.parse_obj({
        **media_updated_event_data,
        "payload": media_event_data_template['payload'],
    })
    assert updated_event_without_previous_attributes.payload.previous_attributes == {}


@pytest.mark.parametrize("test_event_data,expected_event_type", [
    (media_created_event_data, "media.created"),
    (media_processing_event_data, "media.processing"),
    (media_ready_event_data, "media.ready"),
    (media_failed_event_data, "media.failed"),
    (media_updated_event_data, "media.updated"),
    (media_deleted_event_data, "media.deleted"),
])
def test_event_parsing(test_event_data, expected_event_type):
    event_delivery_data = {**delivery_template, "events": [test_event_data]}
    delivery = parse_webhook_event_delivery(event_delivery_data)
    assert delivery.events[0].type == expected_event_type


"""
hook=HookInfo(uuid=UUID('a4ab9eb6-ab82-4dae-86f2-29f744f7d031'))
events=[
    MediaCreatedEvent(
        type='media.created', 
        uuid='fc53f8f78b67d04d455029813f8ec1ef', 
        payload=MediaPayload(
            media=MediaInfo(
                id='vpe2p82q64', 
                name='Lenny Delivers Video!', 
                url='https://harper.wistia.com/medias/vpe2p82q64', 
                duration=40.207, 
                thumbnail=Thumbnail(
                    url='http://embed.wistia.com/deliveries/e96ea382701bc172657d90bf269211beddbf6f7f.jpg?image_crop_resized=200x120'
                )
            )
        ),
        metadata={'account_id': '0sxav1wj8o'}, 
        generated_at=datetime.datetime(2020, 3, 31, 21, 56, 45, tzinfo=datetime.timezone.utc)
    )
]
"""


viewing_session_event_data_template = {
    "uuid": "09e7b27100b3a000",
    "type": "viewing_session.play",
    "payload": {
        "visitor": {"id": "v20150227_c651bc81-8ec8-445b-b8a0-878d19278d35"},
        "viewing_session": {"id": "v20150225_e17a7db7-da7d-4d56-b5a8-ce6a9b62a800"},
        "media": {
            "id": "l9dqljgtfy",
            "name": "Lenny Eating Peanuts",
            "url": "http://dave.wistia.com/medias/l9dqljgtfy",
            "duration": 71.912,
            "thumbnail": {
                "url": "http://embed.wistia.com/deliveries/e04d7729a8b83f4fdd52e107b980f5594359bb45.jpg?image_crop_resized=200x120"
            },
        },
    },
    "metadata": {"account_id": "8lq25o0p9c"},
    "generated_at": "2016-03-31T13:59:22Z",
}

viewing_session_play_event_data = {
    **viewing_session_event_data_template,
    "type": "viewing_session.play",
}

viewing_session_percent_watched_event_data = {
    **viewing_session_event_data_template,
    "type": "viewing_session.percent_watched",
    "payload": {
        **viewing_session_event_data_template['payload'],
        "percent_watched": 25,
    },
}

viewing_session_turnstile_converted_event_data = {
    **viewing_session_event_data_template,
    "type": "viewing_session.turnstile.converted",
    "payload": {
        **viewing_session_event_data_template['payload'],
        "name": "Lenny Lavigne",
        "email": "lenny@wistia.com",
    },
}

viewing_session_call_to_action_converted_event_data = {
    **viewing_session_event_data_template,
    "type": "viewing_session.call_to_action.converted",
}

viewing_session_annotation_converted_event_data = {
    **viewing_session_event_data_template,
    "type": "viewing_session.annotation.converted",
}
